from algorithms import ALGORITHMS


def build(setup_kwargs):
    setup_kwargs["setup_requires"] = setup_kwargs["install_requires"]
    setup_kwargs["cffi_modules"] = [f"compile.py:{algorithm}_ffi" for algorithm in ALGORITHMS]



import platform
from cffi import FFI
from pathlib import Path
from algorithms import ALGORITHMS

PATH_ROOT = Path(__file__).parent
PATH_SOURCES = PATH_ROOT / "sources"
PATH_COMMON = PATH_SOURCES / "common"
IS_WINDOWS = "Windows" in platform.system()


DEFINITIONS_KEM = """
    int crypto_kem_keypair(uint8_t *pk, uint8_t *sk);
    int crypto_kem_enc(uint8_t *c, uint8_t *key, const uint8_t *pk);
    int crypto_kem_dec(uint8_t *key, const uint8_t *c, const uint8_t *sk);
    
    #define CRYPTO_PUBLICKEYBYTES ...
    #define CRYPTO_SECRETKEYBYTES ...
    #define CRYPTO_CIPHERTEXTBYTES ...
    #define CRYPTO_BYTES ...
"""

DEFINITIONS_SIGN = """
    int crypto_sign_keypair(uint8_t *pk, uint8_t *sk);
    int crypto_sign_signature(uint8_t *sig, size_t *siglen, const uint8_t *m, size_t mlen, const uint8_t *sk);
    int crypto_sign_verify(const uint8_t *sig, size_t siglen, const uint8_t *m, size_t mlen, const uint8_t *pk);
    int crypto_sign(uint8_t *sm, size_t *smlen, const uint8_t *m, size_t mlen, const uint8_t *sk);
    int crypto_sign_open(uint8_t *m, size_t *mlen, const uint8_t *sm, size_t smlen, const uint8_t *pk);
    
    #define CRYPTO_PUBLICKEYBYTES ...
    #define CRYPTO_SECRETKEYBYTES ...
    #define CRYPTO_BYTES ...
"""


def create_algorithm_ffi(algorithm, *, path, is_kem=False, is_sign=False, **kwargs):
    if not (is_kem ^ is_sign):
        raise ValueError("Algorithm must be KEM or signature scheme")

    algorithm_path = PATH_SOURCES / path
    compiler_args, linker_args, libraries = [], [], []
    variant = "ref"

    if IS_WINDOWS:
        compiler_args += ["/O2", "/nologo"]
        linker_args.append("/NODEFAULTLIB:MSVCRTD")
        libraries.append("advapi32")
    else:
        compiler_args += ["-O3", "-std=c99"]

    if (algorithm_path / "opt").is_dir():
        variant = "opt"

    if (algorithm_path / "vec").is_dir():
        variant = "vec"

    variant_path = algorithm_path / variant
    header_path = variant_path / "api.h"

    ffi = FFI()
    ffi.cdef(DEFINITIONS_KEM if is_kem else DEFINITIONS_SIGN)

    variant_sources = [file for file in variant_path.glob("**/*") if file.is_file() and file.name.endswith(".c")]
    common_sources = [file for file in PATH_COMMON.glob("**/*") if file.is_file() and file.name.endswith(".c")]

    ffi.set_source(
        f"pqcrypto.{'_kem' if is_kem else '_sign'}.{algorithm}",
        f'#include "{ str(header_path.resolve()) }"',
        sources=[str(source.resolve()) for source in (*common_sources, *variant_sources)],
        include_dirs=[str(PATH_COMMON)],
        extra_compile_args=compiler_args,
        extra_link_args=linker_args,
        libraries=libraries,
    )

    return ffi


for algorithm, kwargs in ALGORITHMS.items():
    algorithm_ffi = create_algorithm_ffi(algorithm, **kwargs)
    globals()[f"{algorithm}_ffi"] = algorithm_ffi


if __name__ == "__main__":
    for algorithm in ALGORITHMS:
        globals()[f"{algorithm}_ffi"].compile(verbose=True)



from .._kem.ntruhps2048677 import ffi as __ffi, lib as __lib
from .common import _kem_generate_keypair_factory, _kem_encrypt_factory, _kem_decrypt_factory

PUBLIC_KEY_SIZE = __lib.CRYPTO_PUBLICKEYBYTES
SECRET_KEY_SIZE = __lib.CRYPTO_SECRETKEYBYTES
CIPHERTEXT_SIZE = __lib.CRYPTO_CIPHERTEXTBYTES
PLAINTEXT_SIZE = __lib.CRYPTO_BYTES

generate_keypair = _kem_generate_keypair_factory(__ffi, __lib)
encrypt = _kem_encrypt_factory(__ffi, __lib)
decrypt = _kem_decrypt_factory(__ffi, __lib)



from .._sign.sphincs_sha256_192f_simple import ffi as __ffi, lib as __lib
from .common import _sign_generate_keypair_factory, _sign_sign_factory, _sign_verify_factory

PUBLIC_KEY_SIZE = __lib.CRYPTO_PUBLICKEYBYTES
SECRET_KEY_SIZE = __lib.CRYPTO_SECRETKEYBYTES
SIGNATURE_SIZE = __lib.CRYPTO_BYTES

generate_keypair = _sign_generate_keypair_factory(__ffi, __lib)
sign = _sign_sign_factory(__ffi, __lib)
verify = _sign_verify_factory(__ffi, __lib)



from ..common import _run_in_threadpool


def _kem_generate_keypair_factory(ffi, lib, use_threadpool=False):
    def generate_keypair():
        public_key_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_PUBLICKEYBYTES))
        secret_key_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_SECRETKEYBYTES))

        if 0 != lib.crypto_kem_keypair(public_key_buf, secret_key_buf):
            raise RuntimeError("KEM keypair generation failed")

        public_key = bytes(ffi.buffer(public_key_buf, lib.CRYPTO_PUBLICKEYBYTES))
        secret_key = bytes(ffi.buffer(secret_key_buf, lib.CRYPTO_SECRETKEYBYTES))

        return public_key, secret_key

    return _run_in_threadpool(generate_keypair) if use_threadpool else generate_keypair


def _kem_encrypt_factory(ffi, lib, use_threadpool=False):
    def encrypt(public_key):
        if not isinstance(public_key, bytes):
            raise TypeError("'public_key' must be of type 'bytes'")

        if len(public_key) != lib.CRYPTO_PUBLICKEYBYTES:
            raise ValueError(f"'public_key' must be of length '{ lib.CRYPTO_PUBLICKEYBYTES }'")

        ciphertext_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_CIPHERTEXTBYTES))
        plaintext_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_BYTES))

        if 0 != lib.crypto_kem_enc(ciphertext_buf, plaintext_buf, public_key):
            raise RuntimeError("KEM encryption failed")

        ciphertext = bytes(ffi.buffer(ciphertext_buf, lib.CRYPTO_CIPHERTEXTBYTES))
        plaintext = bytes(ffi.buffer(plaintext_buf, lib.CRYPTO_BYTES))

        return ciphertext, plaintext

    return _run_in_threadpool(encrypt) if use_threadpool else encrypt


def _kem_decrypt_factory(ffi, lib, use_threadpool=False):
    def decrypt(secret_key, ciphertext):
        if not isinstance(secret_key, bytes):
            raise TypeError("'secret_key' must be of type 'bytes'")

        if not isinstance(ciphertext, bytes):
            raise TypeError("'ciphertext' must be of type 'bytes'")

        if len(secret_key) != lib.CRYPTO_SECRETKEYBYTES:
            raise ValueError(f"'secret_key' must be of length '{ lib.CRYPTO_SECRETKEYBYTES }'")

        if len(ciphertext) != lib.CRYPTO_CIPHERTEXTBYTES:
            raise ValueError(f"'ciphertext' must be of length '{ lib.CRYPTO_CIPHERTEXTBYTES }'")

        plaintext_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_BYTES))

        if 0 != lib.crypto_kem_dec(plaintext_buf, ciphertext, secret_key):
            raise RuntimeError("KEM decryption failed")

        return bytes(ffi.buffer(plaintext_buf, lib.CRYPTO_BYTES))

    return _run_in_threadpool(decrypt) if use_threadpool else decrypt



ALGORITHMS = {
    "firesaber": {"path": "firesaber", "is_kem": True},
    "frodokem1344aes": {"path": "frodokem1344aes", "is_kem": True},
    "frodokem1344shake": {"path": "frodokem1344shake", "is_kem": True},
    "frodokem640aes": {"path": "frodokem640aes", "is_kem": True},
    "frodokem640shake": {"path": "frodokem640shake", "is_kem": True},
    "frodokem976aes": {"path": "frodokem976aes", "is_kem": True},
    "frodokem976shake": {"path": "frodokem976shake", "is_kem": True},
    "kyber1024": {"path": "kyber1024", "is_kem": True},
    "kyber1024_90s": {"path": "kyber1024-90s", "is_kem": True},
    "kyber512": {"path": "kyber512", "is_kem": True},
    "kyber512_90s": {"path": "kyber512-90s", "is_kem": True},
    "kyber768": {"path": "kyber768", "is_kem": True},
    "kyber768_90s": {"path": "kyber768-90s", "is_kem": True},
    "lightsaber": {"path": "lightsaber", "is_kem": True},
    "mceliece348864": {"path": "mceliece348864", "is_kem": True, "generate_in_threadpool": True},
    "mceliece348864f": {"path": "mceliece348864f", "is_kem": True, "generate_in_threadpool": True},
    "mceliece460896": {"path": "mceliece460896", "is_kem": True, "generate_in_threadpool": True},
    "mceliece460896f": {"path": "mceliece460896f", "is_kem": True, "generate_in_threadpool": True},
    "mceliece6688128": {"path": "mceliece6688128", "is_kem": True, "generate_in_threadpool": True},
    "mceliece6688128f": {"path": "mceliece6688128f", "is_kem": True, "generate_in_threadpool": True},
    "mceliece6960119": {"path": "mceliece6960119", "is_kem": True, "generate_in_threadpool": True},
    "mceliece6960119f": {"path": "mceliece6960119f", "is_kem": True, "generate_in_threadpool": True},
    "mceliece8192128": {"path": "mceliece8192128", "is_kem": True, "generate_in_threadpool": True},
    "mceliece8192128f": {"path": "mceliece8192128f", "is_kem": True, "generate_in_threadpool": True},
    "ntruhps2048509": {"path": "ntruhps2048509", "is_kem": True},
    "ntruhps2048677": {"path": "ntruhps2048677", "is_kem": True},
    "ntruhps4096821": {"path": "ntruhps4096821", "is_kem": True},
    "ntruhrss701": {"path": "ntruhrss701", "is_kem": True},
    "saber": {"path": "saber", "is_kem": True},
    "dilithium2": {"path": "dilithium2", "is_sign": True},
    "dilithium3": {"path": "dilithium3", "is_sign": True},
    "dilithium4": {"path": "dilithium4", "is_sign": True},
    "falcon_1024": {"path": "falcon-1024", "is_sign": True},
    "falcon_512": {"path": "falcon-512", "is_sign": True},
    "rainbowIa_classic": {
        "path": "rainbowIa-classic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowIa_cyclic": {
        "path": "rainbowIa-cyclic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowIa_cyclic_compressed": {
        "path": "rainbowIa-cyclic-compressed",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowIIIc_classic": {
        "path": "rainbowIIIc-classic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowIIIc_cyclic": {
        "path": "rainbowIIIc-cyclic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowIIIc_cyclic_compressed": {
        "path": "rainbowIIIc-cyclic-compressed",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowVc_classic": {
        "path": "rainbowVc-classic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowVc_cyclic": {
        "path": "rainbowVc-cyclic",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "rainbowVc_cyclic_compressed": {
        "path": "rainbowVc-cyclic-compressed",
        "is_sign": True,
        "generate_in_threadpool": True,
        "sign_in_threadpool": True,
        "verify_in_threadpool": True,
    },
    "sphincs_haraka_128f_robust": {"path": "sphincs-haraka-128f-robust", "is_sign": True},
    "sphincs_haraka_128f_simple": {"path": "sphincs-haraka-128f-simple", "is_sign": True},
    "sphincs_haraka_128s_robust": {"path": "sphincs-haraka-128s-robust", "is_sign": True},
    "sphincs_haraka_128s_simple": {"path": "sphincs-haraka-128s-simple", "is_sign": True},
    "sphincs_haraka_192f_robust": {"path": "sphincs-haraka-192f-robust", "is_sign": True},
    "sphincs_haraka_192f_simple": {"path": "sphincs-haraka-192f-simple", "is_sign": True},
    "sphincs_haraka_192s_robust": {"path": "sphincs-haraka-192s-robust", "is_sign": True},
    "sphincs_haraka_192s_simple": {"path": "sphincs-haraka-192s-simple", "is_sign": True},
    "sphincs_haraka_256f_robust": {"path": "sphincs-haraka-256f-robust", "is_sign": True},
    "sphincs_haraka_256f_simple": {"path": "sphincs-haraka-256f-simple", "is_sign": True},
    "sphincs_haraka_256s_robust": {"path": "sphincs-haraka-256s-robust", "is_sign": True},
    "sphincs_haraka_256s_simple": {"path": "sphincs-haraka-256s-simple", "is_sign": True},
    "sphincs_sha256_128f_robust": {"path": "sphincs-sha256-128f-robust", "is_sign": True},
    "sphincs_sha256_128f_simple": {"path": "sphincs-sha256-128f-simple", "is_sign": True},
    "sphincs_sha256_128s_robust": {"path": "sphincs-sha256-128s-robust", "is_sign": True},
    "sphincs_sha256_128s_simple": {"path": "sphincs-sha256-128s-simple", "is_sign": True},
    "sphincs_sha256_192f_robust": {"path": "sphincs-sha256-192f-robust", "is_sign": True},
    "sphincs_sha256_192f_simple": {"path": "sphincs-sha256-192f-simple", "is_sign": True},
    "sphincs_sha256_192s_robust": {"path": "sphincs-sha256-192s-robust", "is_sign": True},
    "sphincs_sha256_192s_simple": {"path": "sphincs-sha256-192s-simple", "is_sign": True},
    "sphincs_sha256_256f_robust": {"path": "sphincs-sha256-256f-robust", "is_sign": True},
    "sphincs_sha256_256f_simple": {"path": "sphincs-sha256-256f-simple", "is_sign": True},
    "sphincs_sha256_256s_robust": {"path": "sphincs-sha256-256s-robust", "is_sign": True},
    "sphincs_sha256_256s_simple": {"path": "sphincs-sha256-256s-simple", "is_sign": True},
    "sphincs_shake256_128f_robust": {"path": "sphincs-shake256-128f-robust", "is_sign": True},
    "sphincs_shake256_128f_simple": {"path": "sphincs-shake256-128f-simple", "is_sign": True},
    "sphincs_shake256_128s_robust": {"path": "sphincs-shake256-128s-robust", "is_sign": True},
    "sphincs_shake256_128s_simple": {"path": "sphincs-shake256-128s-simple", "is_sign": True},
    "sphincs_shake256_192f_robust": {"path": "sphincs-shake256-192f-robust", "is_sign": True},
    "sphincs_shake256_192f_simple": {"path": "sphincs-shake256-192f-simple", "is_sign": True},
    "sphincs_shake256_192s_robust": {"path": "sphincs-shake256-192s-robust", "is_sign": True},
    "sphincs_shake256_192s_simple": {"path": "sphincs-shake256-192s-simple", "is_sign": True},
    "sphincs_shake256_256f_robust": {"path": "sphincs-shake256-256f-robust", "is_sign": True},
    "sphincs_shake256_256f_simple": {"path": "sphincs-shake256-256f-simple", "is_sign": True},
    "sphincs_shake256_256s_robust": {"path": "sphincs-shake256-256s-robust", "is_sign": True},
    "sphincs_shake256_256s_simple": {"path": "sphincs-shake256-256s-simple", "is_sign": True},
}



import threading
import concurrent.futures

_stack_boost = 1024 ** 2 * 10
_stack_extended = False


def _run_in_threadpool(func):
    def _run_in_threadpool_(*args, **kwargs):
        global _stack_extended

        if not _stack_extended:
            stack_size = threading.stack_size() or _stack_boost
            threading.stack_size(stack_size + _stack_boost)
            _stack_extended = True

        with concurrent.futures.ThreadPoolExecutor() as executor:
            future = executor.submit(func, *args, **kwargs)
            return future.result()

    return _run_in_threadpool_



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pqcrypto', 'pqcrypto.kem', 'pqcrypto.sign']

package_data = \
{'': ['*'], 'pqcrypto': ['_kem/*', '_sign/*']}

install_requires = \
['cffi>=1.14.2,<2.0.0']

setup_kwargs = {
    'name': 'pqcrypto',
    'version': '0.1.3',
    'description': 'Post-quantum cryptography for Python.',
    'long_description': '# 👻 Post-Quantum Cryptography (PQCrypto)\n\nIn recent years, there has been a substantial amount of research on quantum computers – machines that exploit quantum mechanical phenomena to solve mathematical problems that are difficult or intractable for conventional computers. If large-scale quantum computers are ever built, they will be able to break many of the public-key cryptosystems currently in use. This would seriously compromise the confidentiality and integrity of digital communications on the Internet and elsewhere. The goal of post-quantum cryptography (also called quantum-resistant cryptography) is to develop cryptographic systems that are secure against both quantum and classical computers, and can interoperate with existing communications protocols and networks. \n\nThis package provides tested, ergonomic **Python 3** CFFI bindings to implementations of a number of algorithms submitted as part of the [Post-Quantum Cryptography Standardization](https://csrc.nist.gov/projects/post-quantum-cryptography/post-quantum-cryptography-standardization) effort by [NIST](https://www.nist.gov/).\n\n# Installation\n\nYou can install this package using `pip` or build it from source using `poetry`:\n\n    # Using pip\n    pip install pqcrypto\n\n    # Using poetry\n    pip install poetry\n    poetry build\n\n# Key Encapsulation\n\n```python\nfrom secrets import compare_digest\n# from pqcrypto.kem.firesaber import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem1344aes import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem1344shake import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem640aes import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem640shake import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem976aes import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.frodokem976shake import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber1024 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber1024_90s import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber512 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber512_90s import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber768 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.kyber768_90s import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.lightsaber import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece348864 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece348864f import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece460896 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece460896f import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece6688128 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece6688128f import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece6960119 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece6960119f import generate_keypair, encrypt, decrypt\nfrom pqcrypto.kem.mceliece8192128 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.mceliece8192128f import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.ntruhps2048509 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.ntruhps2048677 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.ntruhps4096821 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.ntruhrss701 import generate_keypair, encrypt, decrypt\n# from pqcrypto.kem.saber import generate_keypair, encrypt, decrypt\n\n# Alice generates a (public, secret) key pair\npublic_key, secret_key = generate_keypair()\n\n# Bob derives a secret (the plaintext) and encrypts it with Alice\'s public key to produce a ciphertext\nciphertext, plaintext_original = encrypt(public_key)\n\n# Alice decrypts Bob\'s ciphertext to derive the now shared secret\nplaintext_recovered = decrypt(secret_key, ciphertext)\n\n# Compare the original and recovered secrets in constant time\nassert compare_digest(plaintext_original, plaintext_recovered)\n```\n\n# Signing\n\n```python\n# from pqcrypto.sign.dilithium2 import generate_keypair, sign, verify\n# from pqcrypto.sign.dilithium3 import generate_keypair, sign, verify\nfrom pqcrypto.sign.dilithium4 import generate_keypair, sign, verify\n# from pqcrypto.sign.falcon_1024 import generate_keypair, sign, verify\n# from pqcrypto.sign.falcon_512 import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIa_classic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIa_cyclic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIa_cyclic_compressed import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIIIc_classic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIIIc_cyclic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowIIIc_cyclic_compressed import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowVc_classic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowVc_cyclic import generate_keypair, sign, verify\n# from pqcrypto.sign.rainbowVc_cyclic_compressed import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_128f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_128f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_128s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_128s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_192f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_192f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_192s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_192s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_256f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_256f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_256s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_haraka_256s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_128f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_128f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_128s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_128s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_192f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_192f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_192s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_192s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_256f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_256f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_256s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_sha256_256s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_128f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_128f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_128s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_128s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_192f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_192f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_192s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_192s_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_256f_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_256f_simple import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_256s_robust import generate_keypair, sign, verify\n# from pqcrypto.sign.sphincs_shake256_256s_simple import generate_keypair, sign, verify\n\n# Alice generates a (public, secret) key pair\npublic_key, secret_key = generate_keypair()\n\n# Alice signs her message using her secret key\nsignature = sign(secret_key, b"Hello world")\n\n# Bob uses Alice\'s public key to validate her signature\nassert verify(public_key, b"Hello world", signature)\n```\n\n# Credits\n\nThe C implementations used herein are derived from the [PQClean](https://github.com/pqclean/pqclean/) project.\n\n# License\n```text\nBSD 3-Clause License\n\nCopyright (c) 2020, Phil Demetriou\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n* Redistributions of source code must retain the above copyright notice, this\n  list of conditions and the following disclaimer.\n\n* Redistributions in binary form must reproduce the above copyright notice,\n  this list of conditions and the following disclaimer in the documentation\n  and/or other materials provided with the distribution.\n\n* Neither the name of the copyright holder nor the names of its\n  contributors may be used to endorse or promote products derived from\n  this software without specific prior written permission.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n```',
    'author': 'Phil Demetriou',
    'author_email': 'inbox@philonas.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kpdemetriou/pqcrypto',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}
from extend import *
build(setup_kwargs)

setup(**setup_kwargs)



import struct
from ..common import _run_in_threadpool


def _sign_generate_keypair_factory(ffi, lib, use_threadpool=False):
    def generate_keypair():
        public_key_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_PUBLICKEYBYTES))
        secret_key_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_SECRETKEYBYTES))

        if 0 != lib.crypto_sign_keypair(public_key_buf, secret_key_buf):
            raise RuntimeError("Keypair generation failed")

        public_key = bytes(ffi.buffer(public_key_buf, lib.CRYPTO_PUBLICKEYBYTES))
        secret_key = bytes(ffi.buffer(secret_key_buf, lib.CRYPTO_SECRETKEYBYTES))

        return public_key, secret_key

    return _run_in_threadpool(generate_keypair) if use_threadpool else generate_keypair


def _sign_sign_factory(ffi, lib, use_threadpool=False):
    def sign(secret_key, message):
        if not isinstance(secret_key, bytes):
            raise TypeError("'secret_key' must be of type 'bytes'")

        if not isinstance(message, bytes):
            raise TypeError("'message' must be of type 'bytes'")

        if len(secret_key) != lib.CRYPTO_SECRETKEYBYTES:
            raise ValueError(f"'secret_key' must be of length '{ lib.CRYPTO_SECRETKEYBYTES }'")

        signature_buf = ffi.new("uint8_t [{}]".format(lib.CRYPTO_BYTES))
        signature_len = ffi.new("size_t *", lib.CRYPTO_BYTES)

        if 0 != lib.crypto_sign_signature(signature_buf, signature_len, message, len(message), secret_key):
            raise RuntimeError("Signature generation failed")

        signature_len = struct.unpack("Q", ffi.buffer(signature_len, 8))[0]
        return bytes(ffi.buffer(signature_buf, signature_len))

    return _run_in_threadpool(sign) if use_threadpool else sign


def _sign_verify_factory(ffi, lib, use_threadpool=False):
    def verify(public_key, message, signature):
        if not isinstance(public_key, bytes):
            raise TypeError("'public_key' must be of type 'bytes'")

        if not isinstance(message, bytes):
            raise TypeError("'message' must be of type 'bytes'")

        if not isinstance(signature, bytes):
            raise TypeError("'signature' must be of type 'bytes'")

        if len(public_key) != lib.CRYPTO_PUBLICKEYBYTES:
            raise ValueError(f"'public_key' must be of length '{ lib.CRYPTO_PUBLICKEYBYTES }'")

        if len(signature) > lib.CRYPTO_BYTES:
            raise ValueError(f"'signature' must be of length at most '{ lib.CRYPTO_BYTES }'")

        return 0 == lib.crypto_sign_verify(signature, len(signature), message, len(message), public_key)

    return _run_in_threadpool(verify) if use_threadpool else verify
